@@ -20,6 +20,12 @@ run_test 'top_make testlint' testlint
 
 exit_suite --continue
 
+enter_suite 'lualint'
+
+run_test 'top_make lualint' lualint
+
+exit_suite --continue
+
 enter_suite single-includes
 
 CLICOLOR_FORCE=1 run_test_wd \
