@@ -83,14 +83,16 @@ static inline void restore_win_for_buf(win_T *save_curwin,
   }
 }
 
-static inline void buf_set_changedtick(buf_T *const buf, const int changedtick)
+static inline void buf_set_changedtick(buf_T *const buf,
+                                       const varnumber_T changedtick)
   REAL_FATTR_NONNULL_ALL REAL_FATTR_ALWAYS_INLINE;
 
 /// Set b_changedtick and corresponding variable
 ///
 /// @param[out]  buf  Buffer to set changedtick in.
 /// @param[in]  changedtick  New value.
-static inline void buf_set_changedtick(buf_T *const buf, const int changedtick)
+static inline void buf_set_changedtick(buf_T *const buf,
+                                       const varnumber_T changedtick)
 {
 #ifndef NDEBUG
   dictitem_T *const changedtick_di = tv_dict_find(
@@ -113,7 +115,7 @@ static inline void buf_set_changedtick(buf_T *const buf, const int changedtick)
   do { \
     win_T *save_curwin = NULL; \
     tabpage_T *save_curtab = NULL; \
-    bufref_T save_curbuf = { NULL, 0 }; \
+    bufref_T save_curbuf = { NULL, 0, 0 }; \
     switch_to_win_for_buf(b, &save_curwin, &save_curtab, &save_curbuf); \
     code; \
     restore_win_for_buf(save_curwin, save_curtab, &save_curbuf); \
