@@ -3,23 +3,32 @@
 
 #include "nvim/pos.h"
 #include "nvim/lib/kvec.h"
+#include "nvim/lib/kbtree.h"
+
 // bufhl: buffer specific highlighting
 
-struct bufhl_hl_item
-{
+typedef struct {
   int src_id;
   int hl_id;  // highlight group
   colnr_T start;  // first column to highlight
   colnr_T stop;  // last column to highlight
-};
-typedef struct bufhl_hl_item bufhl_hl_item_T;
+} BufhlItem;
 
-typedef kvec_t(struct bufhl_hl_item) bufhl_vec_T;
+typedef kvec_t(BufhlItem) BufhlItemVec;
 
 typedef struct {
-  bufhl_vec_T entries;
+  linenr_T line;
+  BufhlItemVec items;
+} BufhlLine;
+#define BUFHLLINE_INIT(l) { l, KV_INITIAL_VALUE }
+
+typedef struct {
+  BufhlItemVec entries;
   int current;
   colnr_T valid_to;
-} bufhl_lineinfo_T;
+} BufhlLineInfo;
 
+#define BUFHL_CMP(a, b) ((int)(((a)->line - (b)->line)))
+KBTREE_INIT(bufhl, BufhlLine *, BUFHL_CMP, 10)  // -V512
+typedef kbtree_t(bufhl) BufhlInfo;
 #endif  // NVIM_BUFHL_DEFS_H
