@@ -1210,7 +1210,8 @@ char *tv_dict_get_string(const dict_T *const d, const char *const key,
 ///
 /// @param[in]  d  Dictionary to get item from.
 /// @param[in]  key  Dictionary key.
-/// @param[in]  numbuf  Numbuf for.
+/// @param[in]  numbuf  Buffer for non-string items converted to strings, at
+///                     least of #NUMBUFLEN length.
 ///
 /// @return NULL if key does not exist, empty string in case of type error,
 ///         string item value otherwise.
@@ -1225,6 +1226,32 @@ const char *tv_dict_get_string_buf(const dict_T *const d, const char *const key,
   return tv_get_string_buf(&di->di_tv, numbuf);
 }
 
+/// Get a string item from a dictionary
+///
+/// @param[in]  d  Dictionary to get item from.
+/// @param[in]  key  Dictionary key.
+/// @param[in]  key_len  Key length.
+/// @param[in]  numbuf  Buffer for non-string items converted to strings, at
+///                     least of #NUMBUFLEN length.
+/// @param[in]  def  Default return when key does not exist.
+///
+/// @return `def` when key does not exist,
+///         NULL in case of type error,
+///         string item value in case of success.
+const char *tv_dict_get_string_buf_chk(const dict_T *const d,
+                                       const char *const key,
+                                       const ptrdiff_t key_len,
+                                       char *const numbuf,
+                                       const char *const def)
+  FUNC_ATTR_WARN_UNUSED_RESULT
+{
+  const dictitem_T *const di = tv_dict_find(d, key, key_len);
+  if (di == NULL) {
+    return def;
+  }
+  return tv_get_string_buf_chk(&di->di_tv, numbuf);
+}
+
 /// Get a function from a dictionary
 ///
 /// @param[in]  d  Dictionary to get callback from.
@@ -1359,12 +1386,33 @@ int tv_dict_add_str(dict_T *const d,
                     const char *const key, const size_t key_len,
                     const char *const val)
   FUNC_ATTR_NONNULL_ALL
+{
+  return tv_dict_add_allocated_str(d, key, key_len, xstrdup(val));
+}
+
+/// Add a string entry to dictionary
+///
+/// Unlike tv_dict_add_str() saves val to the new dictionary item in place of
+/// creating a new copy.
+///
+/// @warning String will be freed even in case addition fails.
+///
+/// @param[out]  d  Dictionary to add entry to.
+/// @param[in]  key  Key to add.
+/// @param[in]  key_len  Key length.
+/// @param[in]  val  String to add.
+///
+/// @return OK in case of success, FAIL when key already exists.
+int tv_dict_add_allocated_str(dict_T *const d,
+                              const char *const key, const size_t key_len,
+                              char *const val)
+  FUNC_ATTR_NONNULL_ALL
 {
   dictitem_T *const item = tv_dict_item_alloc_len(key, key_len);
 
   item->di_tv.v_lock = VAR_UNLOCKED;
   item->di_tv.v_type = VAR_STRING;
-  item->di_tv.vval.v_string = (char_u *)xstrdup(val);
+  item->di_tv.vval.v_string = (char_u *)val;
   if (tv_dict_add(d, item) == FAIL) {
     tv_dict_item_free(item);
     return FAIL;
@@ -1869,7 +1917,7 @@ void tv_free(typval_T *tv)
       }
       case VAR_FUNC: {
         func_unref(tv->vval.v_string);
-        // FALLTHROUGH
+        FALLTHROUGH;
       }
       case VAR_STRING: {
         xfree(tv->vval.v_string);
@@ -2378,9 +2426,7 @@ varnumber_T tv_get_number_chk(const typval_T *const tv, bool *const ret_error)
     case VAR_STRING: {
       varnumber_T n = 0;
       if (tv->vval.v_string != NULL) {
-        long nr;
-        vim_str2nr(tv->vval.v_string, NULL, NULL, STR2NR_ALL, &nr, NULL, 0);
-        n = (varnumber_T)nr;
+        vim_str2nr(tv->vval.v_string, NULL, NULL, STR2NR_ALL, &n, NULL, 0);
       }
       return n;
     }
@@ -2417,7 +2463,7 @@ varnumber_T tv_get_number_chk(const typval_T *const tv, bool *const ret_error)
 linenr_T tv_get_lnum(const typval_T *const tv)
   FUNC_ATTR_NONNULL_ALL FUNC_ATTR_WARN_UNUSED_RESULT
 {
-  linenr_T lnum = tv_get_number_chk(tv, NULL);
+  linenr_T lnum = (linenr_T)tv_get_number_chk(tv, NULL);
   if (lnum == 0) {  // No valid number, try using same function as line() does.
     int fnum;
     pos_T *const fp = var2fpos(tv, true, &fnum);
