@@ -7,7 +7,9 @@
 
 #include <uv.h>
 
+#include "nvim/log.h"
 #include "nvim/rbuffer.h"
+#include "nvim/macros.h"
 #include "nvim/event/stream.h"
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
@@ -26,8 +28,9 @@ int stream_set_blocking(int fd, bool blocking)
   uv_loop_init(&loop);
   uv_pipe_init(&loop, &stream, 0);
   uv_pipe_open(&stream, fd);
-  int retval = uv_stream_set_blocking((uv_stream_t *)&stream, blocking);
-  uv_close((uv_handle_t *)&stream, NULL);
+  int retval = uv_stream_set_blocking(STRUCT_CAST(uv_stream_t, &stream),
+                                      blocking);
+  uv_close(STRUCT_CAST(uv_handle_t, &stream), NULL);
   uv_run(&loop, UV_RUN_NOWAIT);  // not necessary, but couldn't hurt.
   uv_loop_close(&loop);
   return retval;
@@ -52,7 +55,7 @@ void stream_init(Loop *loop, Stream *stream, int fd, uv_stream_t *uvstream)
       assert(type == UV_NAMED_PIPE || type == UV_TTY);
       uv_pipe_init(&loop->uv, &stream->uv.pipe, 0);
       uv_pipe_open(&stream->uv.pipe, fd);
-      stream->uvstream = (uv_stream_t *)&stream->uv.pipe;
+      stream->uvstream = STRUCT_CAST(uv_stream_t, &stream->uv.pipe);
     }
   }
 
@@ -79,6 +82,7 @@ void stream_close(Stream *stream, stream_close_cb on_stream_close, void *data)
   FUNC_ATTR_NONNULL_ARG(1)
 {
   assert(!stream->closed);
+  DLOG("closing Stream: %p", stream);
   stream->closed = true;
   stream->close_cb = on_stream_close;
   stream->close_cb_data = data;
