@@ -2894,7 +2894,7 @@ static void foldlevelExpr(fline_T *flp)
   /* KeyTyped may be reset to 0 when calling a function which invokes
    * do_cmdline().  To make 'foldopen' work correctly restore KeyTyped. */
   save_keytyped = KeyTyped;
-  n = eval_foldexpr(flp->wp->w_p_fde, &c);
+  n = (int)eval_foldexpr(flp->wp->w_p_fde, &c);
   KeyTyped = save_keytyped;
 
   switch (c) {
