@@ -5,12 +5,15 @@
 #include "nvim/buffer_defs.h"
 #include "nvim/ex_cmds_defs.h"
 
-/* Values for "noremap" argument of ins_typebuf().  Also used for
- * map->m_noremap and menu->noremap[]. */
-#define REMAP_YES       0       /* allow remapping */
-#define REMAP_NONE      -1      /* no remapping */
-#define REMAP_SCRIPT    -2      /* remap script-local mappings only */
-#define REMAP_SKIP      -3      /* no remapping for first char */
+/// Values for "noremap" argument of ins_typebuf().  Also used for
+/// map->m_noremap and menu->noremap[].
+/// @addtogroup REMAP_VALUES
+/// @{
+#define REMAP_YES       0       ///< allow remapping
+#define REMAP_NONE      -1      ///< no remapping
+#define REMAP_SCRIPT    -2      ///< remap script-local mappings only
+#define REMAP_SKIP      -3      ///< no remapping for first char
+/// @}
 
 #define KEYLEN_PART_KEY -1      /* keylen value for incomplete key-code */
 #define KEYLEN_PART_MAP -2      /* keylen value for incomplete mapping */
