@@ -7,6 +7,11 @@
 #include <string.h>
 #include <stdbool.h>
 
+#ifdef WIN32
+# include <wchar.h>
+# include <winnls.h>
+#endif
+
 #include <msgpack.h>
 
 #include "nvim/ascii.h"
@@ -57,6 +62,7 @@
 #include "nvim/os/input.h"
 #include "nvim/os/os.h"
 #include "nvim/os/time.h"
+#include "nvim/os/fileio.h"
 #include "nvim/event/loop.h"
 #include "nvim/os/signal.h"
 #include "nvim/event/process.h"
@@ -97,10 +103,8 @@ typedef struct {
   bool input_isatty;                    // stdin is a terminal
   bool output_isatty;                   // stdout is a terminal
   bool err_isatty;                      // stderr is a terminal
-  bool headless;                        // Dont try to start an user interface
-                                        // or read/write to stdio(unless
-                                        // embedding)
-  int no_swap_file;                     /* "-n" argument used */
+  bool headless;                        // Do not start the builtin UI.
+  int no_swap_file;                     // "-n" argument used
   int use_debug_break_level;
   int window_count;                     /* number of windows to use */
   int window_layout;                    /* 0, WIN_HOR, WIN_VER or WIN_TABS */
@@ -124,7 +128,7 @@ typedef struct {
 
 Loop main_loop;
 
-static char *argv0;
+static char *argv0 = NULL;
 
 // Error messages
 static const char *err_arg_missing = N_("Argument missing after");
@@ -153,10 +157,11 @@ void event_init(void)
   terminal_init();
 }
 
-void event_teardown(void)
+/// @returns false if main_loop could not be closed gracefully
+bool event_teardown(void)
 {
   if (!main_loop.events) {
-    return;
+    return true;
   }
 
   multiqueue_process_events(main_loop.events);
@@ -168,7 +173,7 @@ void event_teardown(void)
   signal_teardown();
   terminal_teardown();
 
-  loop_close(&main_loop, true);
+  return loop_close(&main_loop, true);
 }
 
 /// Performs early initialization.
@@ -179,11 +184,9 @@ void early_init(void)
   log_init();
   fs_init();
   handle_init();
-
   eval_init();          // init global variables
-
-  // Init the table of Normal mode commands.
-  init_normal_cmds();
+  init_path(argv0 ? argv0 : "nvim");
+  init_normal_cmds();   // Init the table of Normal mode commands.
 
 #if defined(HAVE_LOCALE_H)
   // Setup to use the current locale (for ctype() and many other things).
@@ -217,11 +220,23 @@ void early_init(void)
 
 #ifdef MAKE_LIB
 int nvim_main(int argc, char **argv)
+#elif defined(WIN32)
+int wmain(int argc, wchar_t **argv_w)  // multibyte args on Windows. #7060
 #else
 int main(int argc, char **argv)
 #endif
 {
-  argv0 = (char *)path_tail((char_u *)argv[0]);
+#if defined(WIN32) && !defined(MAKE_LIB)
+  char *argv[argc];
+  for (int i = 0; i < argc; i++) {
+    char *buf = NULL;
+    utf16_to_utf8(argv_w[i], &buf);
+    assert(buf);
+    argv[i] = buf;
+  }
+#endif
+
+  argv0 = argv[0];
 
   char_u *fname = NULL;   // file name from command line
   mparm_T params;         // various parameters passed between
@@ -241,8 +256,6 @@ int main(int argc, char **argv)
   // Check if we have an interactive window.
   check_and_set_isatty(&params);
 
-  init_path(argv[0]);
-
   event_init();
   /*
    * Process the command line arguments.  File names are put in the global
@@ -636,6 +649,11 @@ void getout(int exitval)
   /* Position the cursor again, the autocommands may have moved it */
   ui_cursor_goto((int)Rows - 1, 0);
 
+  // Apply 'titleold'.
+  if (p_title && *p_titleold != NUL) {
+    ui_call_set_title(cstr_as_string((char *)p_titleold));
+  }
+
 #if defined(USE_ICONV) && defined(DYNAMIC_ICONV)
   iconv_end();
 #endif
@@ -660,8 +678,9 @@ void getout(int exitval)
 ///
 /// @return argument's numeric value otherwise
 static int get_number_arg(const char *p, int *idx, int def)
+  FUNC_ATTR_NONNULL_ALL FUNC_ATTR_WARN_UNUSED_RESULT
 {
-  if (ascii_isdigit(p[*idx])) {
+  if (ascii_isdigit(p[*idx])) {  // -V522
     def = atoi(&(p[*idx]));
     while (ascii_isdigit(p[*idx])) {
       *idx = *idx + 1;
@@ -768,16 +787,26 @@ static void command_line_scan(mparm_T *parmp)
             version();
             mch_exit(0);
           } else if (STRICMP(argv[0] + argv_idx, "api-info") == 0) {
-            msgpack_sbuffer* b = msgpack_sbuffer_new();
-            msgpack_packer* p = msgpack_packer_new(b, msgpack_sbuffer_write);
+            FileDescriptor fp;
+            const int fof_ret = file_open_fd(&fp, OS_STDOUT_FILENO, true);
+            msgpack_packer *p = msgpack_packer_new(&fp, msgpack_file_write);
+
+            if (fof_ret != 0) {
+              emsgf(_("E5421: Failed to open stdin: %s"), os_strerror(fof_ret));
+            }
+
+            if (p == NULL) {
+              emsgf(_(e_outofmem));
+            }
+
             Object md = DICTIONARY_OBJ(api_metadata());
             msgpack_rpc_from_object(md, p);
 
-            for (size_t i = 0; i < b->size; i++) {
-              putchar(b->data[i]);
-            }
-
             msgpack_packer_free(p);
+            const int ff_ret = file_flush(&fp);
+            if (ff_ret < 0) {
+              msgpack_file_write_error(ff_ret);
+            }
             mch_exit(0);
           } else if (STRICMP(argv[0] + argv_idx, "headless") == 0) {
             parmp->headless = true;
@@ -923,10 +952,11 @@ static void command_line_scan(mparm_T *parmp)
           break;
 
         case 's':
-          if (exmode_active)              /* "-s" silent (batch) mode */
-            silent_mode = TRUE;
-          else                    /* "-s {scriptin}" read from script file */
-            want_argument = TRUE;
+          if (exmode_active) {    // "-es" silent (batch) mode
+            silent_mode = true;
+          } else {                // "-s {scriptin}" read from script file
+            want_argument = true;
+          }
           break;
 
         case 't':                 /* "-t {tag}" or "-t{tag}" jump to tag */
@@ -1210,20 +1240,24 @@ static void init_startuptime(mparm_T *paramp)
 
 static void check_and_set_isatty(mparm_T *paramp)
 {
-  paramp->input_isatty = os_isatty(fileno(stdin));
-  paramp->output_isatty = os_isatty(fileno(stdout));
+  stdin_isatty
+    = paramp->input_isatty = os_isatty(fileno(stdin));
+  stdout_isatty
+    = paramp->output_isatty = os_isatty(fileno(stdout));
   paramp->err_isatty = os_isatty(fileno(stderr));
   TIME_MSG("window checked");
 }
 
 // Sets v:progname and v:progpath. Also modifies $PATH on Windows.
-static void init_path(char *exename)
+static void init_path(const char *exename)
+  FUNC_ATTR_NONNULL_ALL
 {
   char exepath[MAXPATHL] = { 0 };
   size_t exepathlen = MAXPATHL;
   // Make v:progpath absolute.
   if (os_exepath(exepath, &exepathlen) != 0) {
-    EMSG2(e_intern2, "init_path()");
+    // Fall back to argv[0]. Missing procfs? #6734
+    path_guess_exepath(exename, exepath, sizeof(exepath));
   }
   set_vim_var_string(VV_PROGPATH, exepath, -1);
   set_vim_var_string(VV_PROGNAME, (char *)path_tail((char_u *)exename), -1);
@@ -1281,10 +1315,29 @@ static void set_window_layout(mparm_T *paramp)
 static void load_plugins(void)
 {
   if (p_lpl) {
-    source_runtime((char_u *)"plugin/**/*.vim", DIP_ALL | DIP_NOAFTER);  // NOLINT
-    TIME_MSG("loading plugins");
+    char_u *rtp_copy = NULL;
 
-    ex_packloadall(NULL);
+    // First add all package directories to 'runtimepath', so that their
+    // autoload directories can be found.  Only if not done already with a
+    // :packloadall command.
+    // Make a copy of 'runtimepath', so that source_runtime does not use the
+    // pack directories.
+    if (!did_source_packages) {
+      rtp_copy = vim_strsave(p_rtp);
+      add_pack_start_dirs();
+    }
+
+    source_in_path(rtp_copy == NULL ? p_rtp : rtp_copy,
+                   (char_u *)"plugin/**/*.vim",  // NOLINT
+                   DIP_ALL | DIP_NOAFTER);
+    TIME_MSG("loading plugins");
+    xfree(rtp_copy);
+
+    // Only source "start" packages if not done already with a :packloadall
+    // command.
+    if (!did_source_packages) {
+      load_start_packages();
+    }
     TIME_MSG("loading packages");
 
     source_runtime((char_u *)"plugin/**/*.vim", DIP_ALL | DIP_AFTER);
@@ -1303,8 +1356,8 @@ static void handle_quickfix(mparm_T *paramp)
       set_string_option_direct((char_u *)"ef", -1,
           paramp->use_ef, OPT_FREE, SID_CARG);
     vim_snprintf((char *)IObuff, IOSIZE, "cfile %s", p_ef);
-    if (qf_init(NULL, p_ef, p_efm, TRUE, IObuff) < 0) {
-      ui_putc('\n');
+    if (qf_init(NULL, p_ef, p_efm, true, IObuff) < 0) {
+      ui_linefeed();
       mch_exit(3);
     }
     TIME_MSG("reading errorfile");
@@ -1684,7 +1737,7 @@ static bool do_user_initialization(void)
     do {
       const char *dir;
       size_t dir_len;
-      iter = vim_colon_env_iter(config_dirs, iter, &dir, &dir_len);
+      iter = vim_env_iter(':', config_dirs, iter, &dir, &dir_len);
       if (dir == NULL || dir_len == 0) {
         break;
       }
@@ -1815,6 +1868,7 @@ static int process_env(char *env, bool is_viminit)
 /// os_fileinfo_link() respectively for extra security.
 static bool file_owned(const char *fname)
 {
+  assert(fname != NULL);
   uid_t uid = getuid();
   FileInfo file_info;
   bool file_owned = os_fileinfo(fname, &file_info)
@@ -1833,9 +1887,11 @@ static bool file_owned(const char *fname)
 /// @param str     string to append to the primary error message, or NULL
 static void mainerr(const char *errstr, const char *str)
 {
+  char *prgname = (char *)path_tail((char_u *)argv0);
+
   signal_stop();              // kill us with CTRL-C here, if you like
 
-  mch_errmsg(argv0);
+  mch_errmsg(prgname);
   mch_errmsg(": ");
   mch_errmsg(_(errstr));
   if (str != NULL) {
@@ -1844,7 +1900,7 @@ static void mainerr(const char *errstr, const char *str)
     mch_errmsg("\"");
   }
   mch_errmsg(_("\nMore info with \""));
-  mch_errmsg(argv0);
+  mch_errmsg(prgname);
   mch_errmsg(" -h\"\n");
 
   mch_exit(1);
@@ -1865,54 +1921,47 @@ static void usage(void)
   signal_stop();              // kill us with CTRL-C here, if you like
 
   mch_msg(_("Usage:\n"));
-  mch_msg(_("  nvim [arguments] [file ...]      Edit specified file(s)\n"));
-  mch_msg(_("  nvim [arguments] -               Read text from stdin\n"));
-  mch_msg(_("  nvim [arguments] -t <tag>        Edit file where tag is defined\n"));
-  mch_msg(_("  nvim [arguments] -q [errorfile]  Edit file with first error\n"));
-  mch_msg(_("\nArguments:\n"));
+  mch_msg(_("  nvim [options] [file ...]      Edit file(s)\n"));
+  mch_msg(_("  nvim [options] -               Read text from stdin\n"));
+  mch_msg(_("  nvim [options] -t <tag>        Edit file where tag is defined\n"));
+  mch_msg(_("  nvim [options] -q [errorfile]  Edit file with first error\n"));
+  mch_msg(_("\nOptions:\n"));
   mch_msg(_("  --                    Only file names after this\n"));
+  mch_msg(_("  +                     Start at end of file\n"));
+  mch_msg(_("  --cmd <cmd>           Execute <cmd> before any config\n"));
+  mch_msg(_("  +<cmd>, -c <cmd>      Execute <cmd> after config and first file\n"));
+  mch_msg("\n");
+  mch_msg(_("  -b                    Binary mode\n"));
+  mch_msg(_("  -d                    Diff mode\n"));
+  mch_msg(_("  -e, -E                Ex mode, Improved Ex mode\n"));
+  mch_msg(_("  -es                   Silent (batch) mode\n"));
+  mch_msg(_("  -h, --help            Print this help message\n"));
+  mch_msg(_("  -i <shada>            Use this shada file\n"));
+  mch_msg(_("  -m                    Modifications (writing files) not allowed\n"));
+  mch_msg(_("  -M                    Modifications in text not allowed\n"));
+  mch_msg(_("  -n                    No swap file, use memory only\n"));
+  mch_msg(_("  -o[N]                 Open N windows (default: one per file)\n"));
+  mch_msg(_("  -O[N]                 Open N vertical windows (default: one per file)\n"));
+  mch_msg(_("  -p[N]                 Open N tab pages (default: one per file)\n"));
+  mch_msg(_("  -r, -L                List swap files\n"));
+  mch_msg(_("  -r <file>             Recover edit state for this file\n"));
+  mch_msg(_("  -R                    Read-only mode\n"));
+  mch_msg(_("  -S <session>          Source <session> after loading the first file\n"));
+  mch_msg(_("  -s <scriptin>         Read Normal mode commands from <scriptin>\n"));
+  mch_msg(_("  -u <config>           Use this config file\n"));
+  mch_msg(_("  -v, --version         Print version information\n"));
+  mch_msg(_("  -V[N][file]           Verbose [level][file]\n"));
+  mch_msg(_("  -Z                    Restricted mode\n"));
+  mch_msg("\n");
+  mch_msg(_("  --api-info            Write msgpack-encoded API metadata to stdout\n"));
+  mch_msg(_("  --embed               Use stdin/stdout as a msgpack-rpc channel\n"));
+  mch_msg(_("  --headless            Don't start a user interface\n"));
 #if !defined(UNIX)
   mch_msg(_("  --literal             Don't expand wildcards\n"));
 #endif
-  mch_msg(_("  -e                    Ex mode\n"));
-  mch_msg(_("  -E                    Improved Ex mode\n"));
-  mch_msg(_("  -s                    Silent (batch) mode (only for ex mode)\n"));
-  mch_msg(_("  -d                    Diff mode\n"));
-  mch_msg(_("  -R                    Read-only mode\n"));
-  mch_msg(_("  -Z                    Restricted mode\n"));
-  mch_msg(_("  -m                    Modifications (writing files) not allowed\n"));
-  mch_msg(_("  -M                    Modifications in text not allowed\n"));
-  mch_msg(_("  -b                    Binary mode\n"));
-  mch_msg(_("  -l                    Lisp mode\n"));
-  mch_msg(_("  -A                    Arabic mode\n"));
-  mch_msg(_("  -F                    Farsi mode\n"));
-  mch_msg(_("  -H                    Hebrew mode\n"));
-  mch_msg(_("  -V[N][file]           Be verbose [level N][log messages to file]\n"));
-  mch_msg(_("  -D                    Debugging mode\n"));
-  mch_msg(_("  -n                    No swap file, use memory only\n"));
-  mch_msg(_("  -r, -L                List swap files and exit\n"));
-  mch_msg(_("  -r <file>             Recover crashed session\n"));
-  mch_msg(_("  -u <vimrc>            Use <vimrc> instead of the default\n"));
-  mch_msg(_("  -i <shada>            Use <shada> instead of the default\n"));
-  mch_msg(_("  --noplugin            Don't load plugin scripts\n"));
-  mch_msg(_("  -o[N]                 Open N windows (default: one for each file)\n"));
-  mch_msg(_("  -O[N]                 Like -o but split vertically\n"));
-  mch_msg(_("  -p[N]                 Open N tab pages (default: one for each file)\n"));
-  mch_msg(_("  +                     Start at end of file\n"));
-  mch_msg(_("  +<linenum>            Start at line <linenum>\n"));
-  mch_msg(_("  +/<pattern>           Start at first occurrence of <pattern>\n"));
-  mch_msg(_("  --cmd <command>       Execute <command> before loading any vimrc\n"));
-  mch_msg(_("  -c <command>          Execute <command> after loading the first file\n"));
-  mch_msg(_("  -S <session>          Source <session> after loading the first file\n"));
-  mch_msg(_("  -s <scriptin>         Read Normal mode commands from <scriptin>\n"));
-  mch_msg(_("  -w <scriptout>        Append all typed characters to <scriptout>\n"));
-  mch_msg(_("  -W <scriptout>        Write all typed characters to <scriptout>\n"));
+  mch_msg(_("  --noplugin            Don't load plugins\n"));
   mch_msg(_("  --startuptime <file>  Write startup timing messages to <file>\n"));
-  mch_msg(_("  --api-info            Dump API metadata serialized to msgpack and exit\n"));
-  mch_msg(_("  --embed               Use stdin/stdout as a msgpack-rpc channel\n"));
-  mch_msg(_("  --headless            Don't start a user interface\n"));
-  mch_msg(_("  -v, --version         Print version information and exit\n"));
-  mch_msg(_("  -h, --help            Print this help message and exit\n"));
+  mch_msg(_("\nSee \":help startup-options\" for all options.\n"));
 }
 
 
