@@ -13,6 +13,13 @@
 # include "nvim/os/unix_defs.h"
 #endif
 
+/// File descriptor number used for standard IO streams
+enum {
+  OS_STDIN_FILENO = STDIN_FILENO,
+  OS_STDOUT_FILENO = STDOUT_FILENO,
+  OS_STDERR_FILENO = STDERR_FILENO,
+};
+
 #define BASENAMELEN (NAME_MAX - 5)
 
 // Use the system path length if it makes sense.
