@@ -8,7 +8,7 @@
 // POSIX.1-2008 says that NAME_MAX should be in here
 #include <limits.h>
 
-#define TEMP_DIR_NAMES {"$TMPDIR", "/tmp", ".", "~"}
+#define TEMP_DIR_NAMES { "$TMPDIR", "/tmp", ".", "~" }
 #define TEMP_FILE_PATH_MAXLEN 256
 
 #define HAVE_ACL (HAVE_POSIX_ACL || HAVE_SOLARIS_ACL)
@@ -16,7 +16,8 @@
 // Special wildcards that need to be handled by the shell.
 #define SPECIAL_WILDCHAR "`'{"
 
-// Separator character for environment variables.
+// Character that separates entries in $PATH.
 #define ENV_SEPCHAR ':'
+#define ENV_SEPSTR  ":"
 
 #endif  // NVIM_OS_UNIX_DEFS_H
