@@ -98,7 +98,6 @@ redo:
   }
 
   if (pum_external) {
-    Array args = ARRAY_DICT_INIT;
     if (array_changed) {
       Array arr = ARRAY_DICT_INIT;
       for (i = 0; i < size; i++) {
@@ -109,14 +108,9 @@ redo:
         ADD(item, STRING_OBJ(cstr_to_string((char *)array[i].pum_info)));
         ADD(arr, ARRAY_OBJ(item));
       }
-      ADD(args, ARRAY_OBJ(arr));
-      ADD(args, INTEGER_OBJ(selected));
-      ADD(args, INTEGER_OBJ(row));
-      ADD(args, INTEGER_OBJ(col));
-      ui_event("popupmenu_show", args);
+      ui_call_popupmenu_show(arr, selected, row, col);
     } else {
-      ADD(args, INTEGER_OBJ(selected));
-      ui_event("popupmenu_select", args);
+      ui_call_popupmenu_select(selected);
     }
     return;
   }
@@ -313,10 +307,10 @@ void pum_redraw(void)
 {
   int row = pum_row;
   int col;
-  int attr_norm = highlight_attr[HLF_PNI];
-  int attr_select = highlight_attr[HLF_PSI];
-  int attr_scroll = highlight_attr[HLF_PSB];
-  int attr_thumb = highlight_attr[HLF_PST];
+  int attr_norm = win_hl_attr(curwin, HLF_PNI);
+  int attr_select = win_hl_attr(curwin, HLF_PSI);
+  int attr_scroll = win_hl_attr(curwin, HLF_PSB);
+  int attr_thumb = win_hl_attr(curwin, HLF_PST);
   int attr;
   int i;
   int idx;
@@ -713,8 +707,7 @@ void pum_undisplay(void)
   pum_array = NULL;
 
   if (pum_external) {
-    Array args = ARRAY_DICT_INIT;
-    ui_event("popupmenu_hide", args);
+    ui_call_popupmenu_hide();
   } else {
     redraw_all_later(SOME_VALID);
     redraw_tabline = true;
