@@ -1,6 +1,10 @@
 #ifndef NVIM_REGEXP_H
 #define NVIM_REGEXP_H
 
+#include "nvim/types.h"
+#include "nvim/buffer_defs.h"
+#include "nvim/regexp_defs.h"
+
 /* Second argument for vim_regcomp(). */
 #define RE_MAGIC        1       /* 'magic' option */
 #define RE_STRING       2       /* match in string instead of buffer text */
