@@ -6,6 +6,7 @@
 #include "nvim/lib/kvec.h"
 
 #include "nvim/ascii.h"
+#include "nvim/log.h"
 #include "nvim/state.h"
 #include "nvim/vim.h"
 #include "nvim/main.h"
@@ -25,10 +26,11 @@ void state_enter(VimState *s)
     int check_result = s->check ? s->check(s) : 1;
 
     if (!check_result) {
-      break;
+      break;     // Terminate this state.
     } else if (check_result == -1) {
-      continue;
+      continue;  // check() again.
     }
+    // Execute this state.
 
     int key;
 
@@ -47,11 +49,13 @@ getkey:
       ui_flush();
       // Call `os_inchar` directly to block for events or user input without
       // consuming anything from `input_buffer`(os/input.c) or calling the
-      // mapping engine. If an event was put into the queue, we send K_EVENT
-      // directly.
+      // mapping engine.
       (void)os_inchar(NULL, 0, -1, 0);
       input_disable_events();
-      key = !multiqueue_empty(main_loop.events) ? K_EVENT : safe_vgetc();
+      // If an event was put into the queue, we send K_EVENT directly.
+      key = !multiqueue_empty(main_loop.events)
+            ? K_EVENT
+            : safe_vgetc();
     }
 
     if (key == K_EVENT) {
