@@ -8,13 +8,12 @@
 #include "nvim/api/private/helpers.h"
 #include "nvim/ascii.h"
 #include "nvim/main.h"
+#include "nvim/aucmd.h"
 #include "nvim/os/os.h"
 #include "nvim/os/input.h"
 #include "nvim/event/rstream.h"
 
 #define PASTETOGGLE_KEY "<Paste>"
-#define FOCUSGAINED_KEY "<FocusGained>"
-#define FOCUSLOST_KEY   "<FocusLost>"
 #define KEY_BUFFER_SIZE 0xfff
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
@@ -47,7 +46,13 @@ void term_input_init(TermInput *input, Loop *loop)
   int curflags = termkey_get_canonflags(input->tk);
   termkey_set_canonflags(input->tk, curflags | TERMKEY_CANON_DELBS);
   // setup input handle
+#ifdef WIN32
+  uv_tty_init(loop, &input->tty_in, 0, 1);
+  uv_tty_set_mode(&input->tty_in, UV_TTY_MODE_RAW);
+  rstream_init_stream(&input->read_stream, &input->tty_in, 0xfff);
+#else
   rstream_init_fd(loop, &input->read_stream, input->in_fd, 0xfff);
+#endif
   // initialize a timer handle for handling ESC with libtermkey
   time_watcher_init(loop, &input->timer_handle, input);
 }
@@ -274,9 +279,9 @@ static void timer_cb(TimeWatcher *watcher, void *data)
 
 /// Handle focus events.
 ///
-/// If the upcoming sequence of bytes in the input stream matches either the
-/// escape code for focus gained `<ESC>[I` or focus lost `<ESC>[O` then consume
-/// that sequence and push the appropriate event into the input queue
+/// If the upcoming sequence of bytes in the input stream matches the termcode
+/// for "focus gained" or "focus lost", consume that sequence and schedule an
+/// event on the main loop.
 ///
 /// @param input the input stream
 /// @return true iff handle_focus_event consumed some input
@@ -288,11 +293,7 @@ static bool handle_focus_event(TermInput *input)
     // Advance past the sequence
     bool focus_gained = *rbuffer_get(input->read_stream.buffer, 2) == 'I';
     rbuffer_consumed(input->read_stream.buffer, 3);
-    if (focus_gained) {
-      enqueue_input(input, FOCUSGAINED_KEY, sizeof(FOCUSGAINED_KEY) - 1);
-    } else {
-      enqueue_input(input, FOCUSLOST_KEY, sizeof(FOCUSLOST_KEY) - 1);
-    }
+    aucmd_schedule_focusgained(focus_gained);
     return true;
   }
   return false;
