@@ -81,6 +81,36 @@ describe('api', function()
     end)
   end)
 
+  describe('nvim_execute_lua', function()
+    it('works', function()
+      meths.execute_lua('vim.api.nvim_set_var("test", 3)', {})
+      eq(3, meths.get_var('test'))
+
+      eq(17, meths.execute_lua('a, b = ...\nreturn a + b', {10,7}))
+
+      eq(NIL, meths.execute_lua('function xx(a,b)\nreturn a..b\nend',{}))
+      eq("xy", meths.execute_lua('return xx(...)', {'x','y'}))
+    end)
+
+    it('reports errors', function()
+      eq({false, 'Error loading lua: [string "<nvim>"]:1: '..
+                 "'=' expected near '+'"},
+         meth_pcall(meths.execute_lua, 'a+*b', {}))
+
+      eq({false, 'Error loading lua: [string "<nvim>"]:1: '..
+                 "unexpected symbol near '1'"},
+         meth_pcall(meths.execute_lua, '1+2', {}))
+
+      eq({false, 'Error loading lua: [string "<nvim>"]:1: '..
+                 "unexpected symbol"},
+         meth_pcall(meths.execute_lua, 'aa=bb\0', {}))
+
+      eq({false, 'Error executing lua: [string "<nvim>"]:1: '..
+                 "attempt to call global 'bork' (a nil value)"},
+         meth_pcall(meths.execute_lua, 'bork()', {}))
+    end)
+  end)
+
   describe('nvim_input', function()
     it("VimL error: does NOT fail, updates v:errmsg", function()
       local status, _ = pcall(nvim, "input", ":call bogus_fn()<CR>")
@@ -297,26 +327,94 @@ describe('api', function()
         {'nvim_get_mode', {}},
         {'nvim_eval',     {'1'}},
       }
-      eq({{{mode='n', blocking=false},
-          13,
-          {mode='n', blocking=false},  -- TODO: should be blocked=true
-          1},
-        NIL}, meths.call_atomic(req))
+      eq({ { {mode='n', blocking=false},
+             13,
+             {mode='n', blocking=false},  -- TODO: should be blocked=true ?
+             1 },
+           NIL}, meths.call_atomic(req))
       eq({mode='r', blocking=true}, nvim("get_mode"))
     end)
-    -- TODO: bug #6166
     it("during insert-mode map-pending, returns blocking=true #6166", function()
       command("inoremap xx foo")
       nvim("input", "ix")
       eq({mode='i', blocking=true}, nvim("get_mode"))
     end)
-    -- TODO: bug #6166
     it("during normal-mode gU, returns blocking=false #6166", function()
       nvim("input", "gu")
       eq({mode='no', blocking=false}, nvim("get_mode"))
     end)
   end)
 
+  describe('RPC (K_EVENT) #6166', function()
+    it('does not complete ("interrupt") normal-mode operator-pending', function()
+      helpers.insert([[
+        FIRST LINE
+        SECOND LINE]])
+      nvim('input', 'gg')
+      nvim('input', 'gu')
+      -- Make any RPC request (can be non-async: op-pending does not block).
+      nvim('get_current_buf')
+      -- Buffer should not change.
+      helpers.expect([[
+        FIRST LINE
+        SECOND LINE]])
+      -- Now send input to complete the operator.
+      nvim('input', 'j')
+      helpers.expect([[
+        first line
+        second line]])
+    end)
+
+    it('does not complete ("interrupt") `d` #3732', function()
+      local screen = Screen.new(20, 4)
+      screen:attach()
+      command('set listchars=eol:$')
+      command('set list')
+      feed('ia<cr>b<cr>c<cr><Esc>kkk')
+      feed('d')
+      -- Make any RPC request (can be non-async: op-pending does not block).
+      nvim('get_current_buf')
+      screen:expect([[
+       ^a$                  |
+       b$                  |
+       c$                  |
+                           |
+      ]])
+    end)
+
+    it('does not complete ("interrupt") normal-mode map-pending', function()
+      command("nnoremap dd :let g:foo='it worked...'<CR>")
+      helpers.insert([[
+        FIRST LINE
+        SECOND LINE]])
+      nvim('input', 'gg')
+      nvim('input', 'd')
+      -- Make any RPC request (must be async, because map-pending blocks).
+      nvim('get_api_info')
+      -- Send input to complete the mapping.
+      nvim('input', 'd')
+      helpers.expect([[
+        FIRST LINE
+        SECOND LINE]])
+      eq('it worked...', helpers.eval('g:foo'))
+    end)
+    it('does not complete ("interrupt") insert-mode map-pending', function()
+      command('inoremap xx foo')
+      command('set timeoutlen=9999')
+      helpers.insert([[
+        FIRST LINE
+        SECOND LINE]])
+      nvim('input', 'ix')
+      -- Make any RPC request (must be async, because map-pending blocks).
+      nvim('get_api_info')
+      -- Send input to complete the mapping.
+      nvim('input', 'x')
+      helpers.expect([[
+        FIRST LINE
+        SECOND LINfooE]])
+    end)
+  end)
+
   describe('nvim_replace_termcodes', function()
     it('escapes K_SPECIAL as K_SPECIAL KS_SPECIAL KE_FILLER', function()
       eq('\128\254X', helpers.nvim('replace_termcodes', '\128', true, true, true))
@@ -337,6 +435,27 @@ describe('api', function()
       eq('\128\253\44', helpers.nvim('replace_termcodes',
                                      '<LeftMouse>', true, true, true))
     end)
+
+    it('converts keycodes', function()
+      eq('\nx\27x\rx<x', helpers.nvim('replace_termcodes',
+         '<NL>x<Esc>x<CR>x<lt>x', true, true, true))
+    end)
+
+    it('does not convert keycodes if special=false', function()
+      eq('<NL>x<Esc>x<CR>x<lt>x', helpers.nvim('replace_termcodes',
+         '<NL>x<Esc>x<CR>x<lt>x', true, true, false))
+    end)
+
+    it('does not crash when transforming an empty string', function()
+      -- Actually does not test anything, because current code will use NULL for
+      -- an empty string.
+      --
+      -- Problem here is that if String argument has .data in allocated memory
+      -- then `return str` in vim_replace_termcodes body will make Neovim free
+      -- `str.data` twice: once when freeing arguments, then when freeing return
+      -- value.
+      eq('', meths.replace_termcodes('', true, true, true))
+    end)
   end)
 
   describe('nvim_feedkeys', function()
@@ -345,13 +464,13 @@ describe('api', function()
         -- notice the special char(…) \xe2\80\xa6
         nvim('feedkeys', ':let x1="…"\n', '', true)
 
-        -- Both replace_termcodes and feedkeys escape \x80
+        -- Both nvim_replace_termcodes and nvim_feedkeys escape \x80
         local inp = helpers.nvim('replace_termcodes', ':let x2="…"<CR>', true, true, true)
-        nvim('feedkeys', inp, '', true)
+        nvim('feedkeys', inp, '', true)   -- escape_csi=true
 
-        -- Disabling CSI escaping in feedkeys
+        -- nvim_feedkeys with CSI escaping disabled
         inp = helpers.nvim('replace_termcodes', ':let x3="…"<CR>', true, true, true)
-        nvim('feedkeys', inp, '', false)
+        nvim('feedkeys', inp, '', false)  -- escape_csi=false
 
         helpers.stop()
       end
@@ -542,6 +661,36 @@ describe('api', function()
     end)
   end)
 
+  describe('list_runtime_paths', function()
+    it('returns nothing with empty &runtimepath', function()
+      meths.set_option('runtimepath', '')
+      eq({}, meths.list_runtime_paths())
+    end)
+    it('returns single runtimepath', function()
+      meths.set_option('runtimepath', 'a')
+      eq({'a'}, meths.list_runtime_paths())
+    end)
+    it('returns two runtimepaths', function()
+      meths.set_option('runtimepath', 'a,b')
+      eq({'a', 'b'}, meths.list_runtime_paths())
+    end)
+    it('returns empty strings when appropriate', function()
+      meths.set_option('runtimepath', 'a,,b')
+      eq({'a', '', 'b'}, meths.list_runtime_paths())
+      meths.set_option('runtimepath', ',a,b')
+      eq({'', 'a', 'b'}, meths.list_runtime_paths())
+      meths.set_option('runtimepath', 'a,b,')
+      eq({'a', 'b', ''}, meths.list_runtime_paths())
+    end)
+    it('truncates too long paths', function()
+      local long_path = ('/a'):rep(8192)
+      meths.set_option('runtimepath', long_path)
+      local paths_list = meths.list_runtime_paths()
+      neq({long_path}, paths_list)
+      eq({long_path:sub(1, #(paths_list[1]))}, paths_list)
+    end)
+  end)
+
   it('can throw exceptions', function()
     local status, err = pcall(nvim, 'get_option', 'invalid-option')
     eq(false, status)
