@@ -1,9 +1,13 @@
 local helpers = require('test.functional.helpers')(after_each)
 
-local clear = helpers.clear
-local command = helpers.command
 local eq = helpers.eq
 local eval = helpers.eval
+local clear = helpers.clear
+local meths = helpers.meths
+local expect = helpers.expect
+local command = helpers.command
+local exc_exec = helpers.exc_exec
+local curbufmeths = helpers.curbufmeths
 
 describe('autocmds:', function()
   before_each(clear)
@@ -12,8 +16,8 @@ describe('autocmds:', function()
     local expected = {
       'WinLeave',
       'TabLeave',
-      'TabNew',
       'WinEnter',
+      'TabNew',
       'TabEnter',
       'BufLeave',
       'BufEnter'
@@ -33,4 +37,22 @@ describe('autocmds:', function()
   it('v:vim_did_enter is 1 after VimEnter', function()
     eq(1, eval('v:vim_did_enter'))
   end)
+
+  describe('BufLeave autocommand', function()
+    it('can wipe out the buffer created by :edit which triggered autocmd',
+    function()
+      meths.set_option('hidden', true)
+      curbufmeths.set_lines(0, 1, false, {
+        'start of test file xx',
+        'end of test file xx'})
+
+      command('autocmd BufLeave * bwipeout yy')
+      eq('Vim(edit):E143: Autocommands unexpectedly deleted new buffer yy',
+         exc_exec('edit yy'))
+
+      expect([[
+        start of test file xx
+        end of test file xx]])
+    end)
+  end)
 end)
