@@ -9,6 +9,7 @@ local command = helpers.command
 local wait = helpers.wait
 local iswin = helpers.iswin
 local get_pathsep = helpers.get_pathsep
+local nvim_set = helpers.nvim_set
 local Screen = require('test.functional.ui.screen')
 
 describe('jobs', function()
@@ -669,18 +670,19 @@ describe("pty process teardown", function()
     if helpers.pending_win32(pending) then return end
     -- Use a nested nvim (in :term) to test without --headless.
     feed_command(":terminal '"..helpers.nvim_prog
+      .."' -u NONE -i NONE --cmd '"..nvim_set.."' "
       -- Use :term again in the _nested_ nvim to get a PTY process.
       -- Use `sleep` to simulate a long-running child of the PTY.
-      .."' +terminal +'!(sleep 300 &)' +qa")
+      .."+terminal +'!(sleep 300 &)' +qa")
 
     -- Exiting should terminate all descendants (PTY, its children, ...).
     screen:expect([[
-                                    |
+      ^                              |
       [Process exited 0]            |
                                     |
                                     |
                                     |
-      -- TERMINAL --                |
+                                    |
     ]])
   end)
 end)
