@@ -1,5 +1,5 @@
 local helpers = require('test.functional.helpers')(after_each)
-local clear, execute, write_file = helpers.clear, helpers.execute, helpers.write_file
+local clear, command, write_file = helpers.clear, helpers.command, helpers.write_file
 local eq, eval = helpers.eq, helpers.eval
 
 describe("modeline", function()
@@ -12,7 +12,7 @@ describe("modeline", function()
 
   it('does not crash with a large version number', function()
     write_file(tempfile, 'vim100000000000000000000000')
-    execute('e! ' .. tempfile)
+    command('e! ' .. tempfile)
 
     eq(2, eval('1+1'))  -- Still alive?
   end)
