@@ -89,7 +89,7 @@ describe('assert function:', function()
 
     it('should change v:errors when expected is equal to actual', function()
       call('assert_notequal', 'foo', 'foo')
-      expected_errors({"Expected 'foo' differs from 'foo'"})
+      expected_errors({"Expected not equal to 'foo'"})
     end)
   end)
 
@@ -253,6 +253,21 @@ describe('assert function:', function()
         "Expected range 5 - 7, but got 8",
       })
     end)
+
+    it('assert_inrange(1, 1) returns E119', function()
+      eq('Vim(call):E119: Not enough arguments for function: assert_inrange',
+         exc_exec("call assert_inrange(1, 1)"))
+    end)
+  end)
+
+  -- assert_report({msg})
+  describe('assert_report()', function()
+    it('should add a message to v:errors', function()
+      command("call assert_report('something is wrong')")
+      command("call assert_match('something is wrong', v:errors[0])")
+      command('call remove(v:errors, 0)')
+      expected_empty()
+    end)
   end)
 
   -- assert_exception({cmd}, [, {error}])
