@@ -516,6 +516,15 @@ describe('eval', function()
     eq({'item'}, eval("y"))
   end)
 
+  it('sets the unnamed register when the "u" option is passed to setreg', function()
+    command("call setreg('a','a reg', 'cu')")
+    eq("a reg", eval('@"'))
+    command("call setreg('b','b reg', 'cu')")
+    eq("b reg", eval('@"'))
+    command("call setreg('c','c reg', 'c')")
+    eq("b reg", eval('@"'))
+  end)
+
   it('search and expressions', function()
     command('so test_eval_setup.vim')
     command([=[call SetReg('/', ['abc/'])]=])
