@@ -3,10 +3,11 @@ local Screen = require('test.functional.ui.screen')
 local clear, wait, nvim = helpers.clear, helpers.wait, helpers.nvim
 local nvim_dir, source, eq = helpers.nvim_dir, helpers.source, helpers.eq
 local feed_command, eval = helpers.feed_command, helpers.eval
-
-if helpers.pending_win32(pending) then return end
+local retry = helpers.retry
+local iswin = helpers.iswin
 
 describe(':terminal', function()
+  if helpers.pending_win32(pending) then return end
   local screen
 
   before_each(function()
@@ -70,19 +71,19 @@ describe(':terminal (with fake shell)', function()
 
   it('with no argument, acts like termopen()', function()
     terminal_with_fake_shell()
-    wait()
+    retry(3, 4 * screen.timeout, function()
     screen:expect([[
-      ready $                                           |
+      ^ready $                                           |
       [Process exited 0]                                |
                                                         |
-      -- TERMINAL --                                    |
+      :terminal                                         |
     ]])
+    end)
   end)
 
   it("with no argument, and 'shell' is set to empty string", function()
     nvim('set_option', 'shell', '')
     terminal_with_fake_shell()
-    wait()
     screen:expect([[
       ^                                                  |
       ~                                                 |
@@ -94,46 +95,42 @@ describe(':terminal (with fake shell)', function()
   it("with no argument, but 'shell' has arguments, acts like termopen()", function()
     nvim('set_option', 'shell', nvim_dir..'/shell-test -t jeff')
     terminal_with_fake_shell()
-    wait()
     screen:expect([[
-      jeff $                                            |
+      ^jeff $                                            |
       [Process exited 0]                                |
                                                         |
-      -- TERMINAL --                                    |
+      :terminal                                         |
     ]])
   end)
 
   it('executes a given command through the shell', function()
     terminal_with_fake_shell('echo hi')
-    wait()
     screen:expect([[
-      ready $ echo hi                                   |
+      ^ready $ echo hi                                   |
                                                         |
       [Process exited 0]                                |
-      -- TERMINAL --                                    |
+      :terminal echo hi                                 |
     ]])
   end)
 
   it("executes a given command through the shell, when 'shell' has arguments", function()
     nvim('set_option', 'shell', nvim_dir..'/shell-test -t jeff')
     terminal_with_fake_shell('echo hi')
-    wait()
     screen:expect([[
-      jeff $ echo hi                                    |
+      ^jeff $ echo hi                                    |
                                                         |
       [Process exited 0]                                |
-      -- TERMINAL --                                    |
+      :terminal echo hi                                 |
     ]])
   end)
 
   it('allows quotes and slashes', function()
     terminal_with_fake_shell([[echo 'hello' \ "world"]])
-    wait()
     screen:expect([[
-      ready $ echo 'hello' \ "world"                    |
+      ^ready $ echo 'hello' \ "world"                    |
                                                         |
       [Process exited 0]                                |
-      -- TERMINAL --                                    |
+      :terminal echo 'hello' \ "world"                  |
     ]])
   end)
 
@@ -164,27 +161,29 @@ describe(':terminal (with fake shell)', function()
 
   it('works with :find', function()
     terminal_with_fake_shell()
-    wait()
     screen:expect([[
-      ready $                                           |
+      ^ready $                                           |
       [Process exited 0]                                |
                                                         |
-      -- TERMINAL --                                    |
+      :terminal                                         |
     ]])
     eq('term://', string.match(eval('bufname("%")'), "^term://"))
     helpers.feed([[<C-\><C-N>]])
     feed_command([[find */shadacat.py]])
-    eq('scripts/shadacat.py', eval('bufname("%")'))
+    if iswin() then
+      eq('scripts\\shadacat.py', eval('bufname("%")'))
+    else
+      eq('scripts/shadacat.py', eval('bufname("%")'))
+    end
   end)
 
   it('works with gf', function()
     terminal_with_fake_shell([[echo "scripts/shadacat.py"]])
-    wait()
     screen:expect([[
-      ready $ echo "scripts/shadacat.py"                |
+      ^ready $ echo "scripts/shadacat.py"                |
                                                         |
       [Process exited 0]                                |
-      -- TERMINAL --                                    |
+      :terminal echo "scripts/shadacat.py"              |
     ]])
     helpers.feed([[<C-\><C-N>]])
     eq('term://', string.match(eval('bufname("%")'), "^term://"))
