@@ -4,8 +4,6 @@ local clear, eq, eval = helpers.clear, helpers.eq, helpers.eval
 local feed, nvim = helpers.feed, helpers.nvim
 local feed_data = thelpers.feed_data
 
-if helpers.pending_win32(pending) then return end
-
 describe('terminal mouse', function()
   local screen
 
@@ -67,6 +65,7 @@ describe('terminal mouse', function()
       end)
 
       it('will forward mouse clicks to the program', function()
+        if helpers.pending_win32(pending) then return end
         feed('<LeftMouse><1,2>')
         screen:expect([[
           line27                                            |
@@ -80,6 +79,7 @@ describe('terminal mouse', function()
       end)
 
       it('will forward mouse scroll to the program', function()
+        if helpers.pending_win32(pending) then return end
         feed('<ScrollWheelUp><0,0>')
         screen:expect([[
           line27                                            |
@@ -94,13 +94,14 @@ describe('terminal mouse', function()
     end)
 
     describe('with a split window and other buffer', function()
+      if helpers.pending_win32(pending) then return end
       before_each(function()
         feed('<c-\\><c-n>:vsp<cr>')
         screen:expect([[
           line28                   |line28                  |
           line29                   |line29                  |
           line30                   |line30                  |
-          rows: 5, cols: 25        |rows: 5, cols: 25       |
+          rows: 5, cols: 24        |rows: 5, cols: 24       |
           {2:^ }                        |{2: }                       |
           ==========                ==========              |
           :vsp                                              |
@@ -110,7 +111,7 @@ describe('terminal mouse', function()
           {7:  1 }^                     |line28                  |
           {4:~                        }|line29                  |
           {4:~                        }|line30                  |
-          {4:~                        }|rows: 5, cols: 25       |
+          {4:~                        }|rows: 5, cols: 24       |
           {4:~                        }|{2: }                       |
           ==========                ==========              |
           :enew | set number                                |
@@ -120,16 +121,16 @@ describe('terminal mouse', function()
           {7: 27 }line                 |line28                  |
           {7: 28 }line                 |line29                  |
           {7: 29 }line                 |line30                  |
-          {7: 30 }line                 |rows: 5, cols: 25       |
+          {7: 30 }line                 |rows: 5, cols: 24       |
           {7: 31 }^                     |{2: }                       |
           ==========                ==========              |
                                                             |
         ]])
         feed('<c-w>li')
         screen:expect([[
-          {7: 27 }line                 |line29                  |
-          {7: 28 }line                 |line30                  |
-          {7: 29 }line                 |rows: 5, cols: 25       |
+          {7: 27 }line                 |line28                  |
+          {7: 28 }line                 |line29                  |
+          {7: 29 }line                 |line30                  |
           {7: 30 }line                 |rows: 5, cols: 24       |
           {7: 31 }                     |{1: }                       |
           ==========                ==========              |
@@ -139,8 +140,8 @@ describe('terminal mouse', function()
         thelpers.enable_mouse()
         thelpers.feed_data('mouse enabled\n')
         screen:expect([[
-          {7: 27 }line                 |line30                  |
-          {7: 28 }line                 |rows: 5, cols: 25       |
+          {7: 27 }line                 |line29                  |
+          {7: 28 }line                 |line30                  |
           {7: 29 }line                 |rows: 5, cols: 24       |
           {7: 30 }line                 |mouse enabled           |
           {7: 31 }                     |{1: }                       |
@@ -152,8 +153,8 @@ describe('terminal mouse', function()
       it('wont lose focus if another window is scrolled', function()
         feed('<ScrollWheelUp><0,0><ScrollWheelUp><0,0>')
         screen:expect([[
-          {7: 21 }line                 |line30                  |
-          {7: 22 }line                 |rows: 5, cols: 25       |
+          {7: 21 }line                 |line29                  |
+          {7: 22 }line                 |line30                  |
           {7: 23 }line                 |rows: 5, cols: 24       |
           {7: 24 }line                 |mouse enabled           |
           {7: 25 }line                 |{1: }                       |
@@ -162,8 +163,8 @@ describe('terminal mouse', function()
         ]])
         feed('<S-ScrollWheelDown><0,0>')
         screen:expect([[
-          {7: 26 }line                 |line30                  |
-          {7: 27 }line                 |rows: 5, cols: 25       |
+          {7: 26 }line                 |line29                  |
+          {7: 27 }line                 |line30                  |
           {7: 28 }line                 |rows: 5, cols: 24       |
           {7: 29 }line                 |mouse enabled           |
           {7: 30 }line                 |{1: }                       |
@@ -175,8 +176,8 @@ describe('terminal mouse', function()
       it('will lose focus if another window is clicked', function()
         feed('<LeftMouse><5,1>')
         screen:expect([[
-          {7: 27 }line                 |line30                  |
-          {7: 28 }l^ine                 |rows: 5, cols: 25       |
+          {7: 27 }line                 |line29                  |
+          {7: 28 }l^ine                 |line30                  |
           {7: 29 }line                 |rows: 5, cols: 24       |
           {7: 30 }line                 |mouse enabled           |
           {7: 31 }                     |{2: }                       |
