@@ -3,8 +3,9 @@ local Screen = require('test.functional.ui.screen')
 local os = require('os')
 local clear, feed, insert = helpers.clear, helpers.feed, helpers.insert
 local command = helpers.command
-local eval = helpers.eval
+local eval, exc_exec = helpers.eval, helpers.exc_exec
 local feed_command, request, eq = helpers.feed_command, helpers.request, helpers.eq
+local curbufmeths = helpers.curbufmeths
 
 describe('colorscheme compatibility', function()
   before_each(function()
@@ -650,3 +651,415 @@ describe("'listchars' highlight", function()
     ]])
   end)
 end)
+
+describe("'winhighlight' highlight", function()
+  local screen
+
+  before_each(function()
+    clear()
+    screen = Screen.new(20,8)
+    screen:attach()
+    screen:set_default_attr_ids({
+      [0] = {bold=true, foreground=Screen.colors.Blue},
+      [1] = {background = Screen.colors.DarkBlue},
+      [2] = {background = Screen.colors.DarkBlue, bold = true, foreground = Screen.colors.Blue1},
+      [3] = {bold = true, reverse = true},
+      [4] = {reverse = true},
+      [5] = {background = Screen.colors.DarkGreen},
+      [6] = {background = Screen.colors.DarkGreen, bold = true, foreground = Screen.colors.Blue1},
+      [7] = {background = Screen.colors.DarkMagenta},
+      [8] = {background = Screen.colors.DarkMagenta, bold = true, foreground = Screen.colors.Blue1},
+      [9] = {foreground = Screen.colors.Brown},
+      [10] = {foreground = Screen.colors.Brown, background = Screen.colors.DarkBlue},
+      [11] = {background = Screen.colors.DarkBlue, bold = true, reverse = true},
+      [12] = {background = Screen.colors.DarkGreen, reverse = true},
+      [13] = {background = Screen.colors.Magenta4, reverse = true},
+      [14] = {background = Screen.colors.DarkBlue, reverse = true},
+      [15] = {foreground = Screen.colors.Grey100, background = Screen.colors.Red},
+      [16] = {foreground = Screen.colors.Blue1},
+      [17] = {background = Screen.colors.LightRed},
+      [18] = {background = Screen.colors.Gray90},
+      [19] = {foreground = Screen.colors.LightGrey, background = Screen.colors.DarkGray},
+      [20] = {background = Screen.colors.LightGrey, underline = true},
+      [21] = {bold = true},
+      [22] = {bold = true, foreground = Screen.colors.SeaGreen4},
+      [23] = {background = Screen.colors.LightMagenta},
+      [24] = {background = Screen.colors.WebGray},
+    })
+    command("hi Background1 guibg=DarkBlue")
+    command("hi Background2 guibg=DarkGreen")
+  end)
+
+  it('works for background color', function()
+    insert("aa")
+    command("split")
+    command("set winhl=Normal:Background1")
+    screen:expect([[
+      {1:a^a                  }|
+      {2:~                   }|
+      {2:~                   }|
+      {11:[No Name] [+]       }|
+      aa                  |
+      {0:~                   }|
+      {4:[No Name] [+]       }|
+                          |
+    ]])
+
+    command("enew")
+    screen:expect([[
+      {1:^                    }|
+      {2:~                   }|
+      {2:~                   }|
+      {11:[No Name]           }|
+      aa                  |
+      {0:~                   }|
+      {4:[No Name] [+]       }|
+                          |
+    ]])
+  end)
+
+  it('handles invalid values', function()
+    command("set winhl=Normal:Background1")
+    screen:expect([[
+      {1:^                    }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+                          |
+    ]])
+
+    eq('Vim(set):E474: Invalid argument: winhl=xxx:yyy',
+       exc_exec("set winhl=xxx:yyy"))
+    eq('Normal:Background1', eval('&winhl'))
+    screen:expect([[
+      {1:^                    }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+                          |
+    ]])
+  end)
+
+
+  it('works local to the buffer', function()
+    insert("aa")
+    command("split")
+    command("setlocal winhl=Normal:Background1")
+    screen:expect([[
+      {1:a^a                  }|
+      {2:~                   }|
+      {2:~                   }|
+      {11:[No Name] [+]       }|
+      aa                  |
+      {0:~                   }|
+      {4:[No Name] [+]       }|
+                          |
+    ]])
+
+    command("enew")
+    screen:expect([[
+      ^                    |
+      {0:~                   }|
+      {0:~                   }|
+      {3:[No Name]           }|
+      aa                  |
+      {0:~                   }|
+      {4:[No Name] [+]       }|
+                          |
+    ]])
+
+    command("bnext")
+    screen:expect([[
+      {1:^aa                  }|
+      {2:~                   }|
+      {2:~                   }|
+      {11:[No Name] [+]       }|
+      aa                  |
+      {0:~                   }|
+      {4:[No Name] [+]       }|
+      <f 1 --100%-- col 1 |
+    ]])
+  end)
+
+  it('for inactive window background works', function()
+    command("set winhl=Normal:Background1,NormalNC:Background2")
+    -- tests global value is copied across split
+    command("split")
+    screen:expect([[
+      {1:^                    }|
+      {2:~                   }|
+      {2:~                   }|
+      {11:[No Name]           }|
+      {5:                    }|
+      {6:~                   }|
+      {12:[No Name]           }|
+                          |
+    ]])
+
+    feed("<c-w><c-w>")
+    screen:expect([[
+      {5:                    }|
+      {6:~                   }|
+      {6:~                   }|
+      {12:[No Name]           }|
+      {1:^                    }|
+      {2:~                   }|
+      {11:[No Name]           }|
+                          |
+    ]])
+
+    feed("<c-w><c-w>")
+    screen:expect([[
+      {1:^                    }|
+      {2:~                   }|
+      {2:~                   }|
+      {11:[No Name]           }|
+      {5:                    }|
+      {6:~                   }|
+      {12:[No Name]           }|
+                          |
+    ]])
+  end)
+
+  it('works with NormalNC', function()
+    command("hi NormalNC guibg=DarkMagenta")
+    -- tests global value is copied across split
+    command("split")
+    screen:expect([[
+      ^                    |
+      {0:~                   }|
+      {0:~                   }|
+      {3:[No Name]           }|
+      {7:                    }|
+      {8:~                   }|
+      {13:[No Name]           }|
+                          |
+    ]])
+
+    command("wincmd w")
+    screen:expect([[
+      {7:                    }|
+      {8:~                   }|
+      {8:~                   }|
+      {13:[No Name]           }|
+      ^                    |
+      {0:~                   }|
+      {3:[No Name]           }|
+                          |
+    ]])
+
+
+    -- winbg=Normal:... overrides global NormalNC
+    command("set winhl=Normal:Background1")
+    screen:expect([[
+      {7:                    }|
+      {8:~                   }|
+      {8:~                   }|
+      {13:[No Name]           }|
+      {1:^                    }|
+      {2:~                   }|
+      {11:[No Name]           }|
+                          |
+    ]])
+
+    command("wincmd w")
+    screen:expect([[
+      ^                    |
+      {0:~                   }|
+      {0:~                   }|
+      {3:[No Name]           }|
+      {1:                    }|
+      {2:~                   }|
+      {14:[No Name]           }|
+                          |
+    ]])
+
+    command("wincmd w")
+    command("set winhl=Normal:Background1,NormalNC:Background2")
+    screen:expect([[
+      {7:                    }|
+      {8:~                   }|
+      {8:~                   }|
+      {13:[No Name]           }|
+      {1:^                    }|
+      {2:~                   }|
+      {11:[No Name]           }|
+                          |
+    ]])
+
+    command("wincmd w")
+    screen:expect([[
+      ^                    |
+      {0:~                   }|
+      {0:~                   }|
+      {3:[No Name]           }|
+      {5:                    }|
+      {6:~                   }|
+      {12:[No Name]           }|
+                          |
+    ]])
+  end)
+
+  it('background applies also to non-text', function()
+    insert('Lorem ipsum dolor sit amet ')
+    command('set shiftwidth=2')
+    feed('>>')
+    command('set number')
+    command('set breakindent')
+    command('set briopt=shift:5,min:0')
+    command('set list')
+    command('set showbreak=↪')
+    screen:expect([[
+      {9:  1 }  ^Lorem ipsum do|
+      {9:    }       {0:↪}lor sit |
+      {9:    }       {0:↪}amet{0:-}   |
+      {0:~                   }|
+      {0:~                   }|
+      {0:~                   }|
+      {0:~                   }|
+                          |
+    ]])
+
+    command('set winhl=Normal:Background1')
+    screen:expect([[
+      {10:  1 }{1:  ^Lorem ipsum do}|
+      {10:    }{1:       }{2:↪}{1:lor sit }|
+      {10:    }{1:       }{2:↪}{1:amet}{2:-}{1:   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+                          |
+    ]])
+
+    command('set nowrap')
+    command('set listchars+=extends:❯,precedes:❮')
+    feed('3w')
+    screen:expect([[
+      {10:  1 }{2:❮}{1: dolor ^sit ame}{2:❯}|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+      {2:~                   }|
+                          |
+    ]])
+  end)
+
+  it('can override NonText, Conceal and EndOfBuffer', function()
+    curbufmeths.set_lines(0,-1,true, {"raa\000"})
+    command('call matchaddpos("Conceal", [[1,2]], 0, -1, {"conceal": "#"})')
+    command('set cole=2 cocu=nvic')
+    command('split')
+    command('call matchaddpos("Conceal", [[1,2]], 0, -1, {"conceal": "#"})')
+    command('set winhl=SpecialKey:ErrorMsg,EndOfBuffer:Background1,'
+            ..'Conceal:Background2')
+
+    screen:expect([[
+      ^r{5:#}a{15:^@}               |
+      {1:~                   }|
+      {1:~                   }|
+      {3:[No Name] [+]       }|
+      r{19:#}a{16:^@}               |
+      {0:~                   }|
+      {4:[No Name] [+]       }|
+                          |
+    ]])
+  end)
+
+  it('can override LineNr, CursorColumn and ColorColumn', function()
+    insert('very text\nmore text')
+    command('set number')
+    command('set colorcolumn=2')
+    command('set cursorcolumn')
+
+    command('split')
+    command('set winhl=LineNr:Background1,CursorColumn:Background2,'
+            ..'ColorColumn:ErrorMsg')
+    screen:expect([[
+      {1:  1 }v{15:e}ry tex{5:t}       |
+      {1:  2 }m{15:o}re tex^t       |
+      {0:~                   }|
+      {3:[No Name] [+]       }|
+      {9:  1 }v{17:e}ry tex{18:t}       |
+      {9:  2 }m{17:o}re text       |
+      {4:[No Name] [+]       }|
+                          |
+    ]])
+  end)
+
+  it('can override Tabline', function()
+    command('tabnew')
+    command('set winhl=TabLine:Background1,TabLineSel:ErrorMsg')
+
+    screen:expect([[
+      {20: No Name] }{15: No Name]}{20:X}|
+      ^                    |
+      {0:~                   }|
+      {0:~                   }|
+      {0:~                   }|
+      {0:~                   }|
+      {0:~                   }|
+                          |
+    ]])
+    command("tabnext")
+    screen:expect([[
+      {21: No Name] }{1: No Name]}{20:X}|
+      ^                    |
+      {0:~                   }|
+      {0:~                   }|
+      {0:~                   }|
+      {0:~                   }|
+      {0:~                   }|
+                          |
+    ]])
+  end)
+
+  it('can override popupmenu', function()
+    insert('word wording wordy')
+    command('split')
+    command('set winhl=Pmenu:Background1,PmenuSel:Background2,'
+            ..'PmenuSbar:ErrorMsg,PmenuThumb:Normal')
+    screen:expect([[
+      word wording word^y  |
+      {0:~                   }|
+      {0:~                   }|
+      {3:[No Name] [+]       }|
+      word wording wordy  |
+      {0:~                   }|
+      {4:[No Name] [+]       }|
+                          |
+    ]])
+
+    feed('oword<c-x><c-p>')
+    screen:expect([[
+      word wording wordy  |
+      wordy^               |
+      {1:word           }{0:     }|
+      {1:wording        }{3:     }|
+      {5:wordy          }rdy  |
+      wordy               |
+      {4:[No Name] [+]       }|
+      {21:-- }{22:match 1 of 3}     |
+    ]])
+
+    feed('<esc>u<c-w><c-w>oword<c-x><c-p>')
+    screen:expect([[
+      word wording wordy  |
+      wordy               |
+      {23:word           }{0:     }|
+      {23:wording        }{4:     }|
+      {24:wordy          }rdy  |
+      wordy^               |
+      {3:[No Name] [+]       }|
+      {21:-- }{22:match 1 of 3}     |
+    ]])
+  end)
+end)
