@@ -1,13 +1,12 @@
 local helpers = require('test.functional.helpers')(after_each)
 local clear, eval, eq = helpers.clear, helpers.eval, helpers.eq
-local execute, source = helpers.execute, helpers.source
+local exc_exec, source = helpers.exc_exec, helpers.source
 
 describe('viml', function()
   before_each(clear)
 
   it('parses `<SID>` with turkish locale', function()
-    execute('lang ctype tr_TR.UTF-8')
-    if string.find(eval('v:errmsg'), '^E197: ') then
+    if exc_exec('lang ctype tr_TR.UTF-8') ~= 0 then
       pending("Locale tr_TR.UTF-8 not supported")
       return
     end
