@@ -1,4 +1,5 @@
-local helpers = require("test.unit.helpers")
+local helpers = require("test.unit.helpers")(after_each)
+local itp = helpers.gen_itp(it)
 
 local to_cstr = helpers.to_cstr
 local eq      = helpers.eq
@@ -12,23 +13,23 @@ end
 
 describe('check_ff_value', function()
 
-  it('views empty string as valid', function()
+  itp('views empty string as valid', function()
     eq(1, check_ff_value(""))
   end)
 
-  it('views "unix", "dos" and "mac" as valid', function()
+  itp('views "unix", "dos" and "mac" as valid', function()
     eq(1, check_ff_value("unix"))
     eq(1, check_ff_value("dos"))
     eq(1, check_ff_value("mac"))
   end)
 
-  it('views "foo" as invalid', function()
+  itp('views "foo" as invalid', function()
     eq(0, check_ff_value("foo"))
   end)
 end)
 
 describe('get_sts_value', function()
-  it([[returns 'softtabstop' when it is non-negative]], function()
+  itp([[returns 'softtabstop' when it is non-negative]], function()
     globals.curbuf.b_p_sts = 5
     eq(5, option.get_sts_value())
 
@@ -36,7 +37,7 @@ describe('get_sts_value', function()
     eq(0, option.get_sts_value())
   end)
 
-  it([[returns "effective shiftwidth" when 'softtabstop' is negative]], function()
+  itp([[returns "effective shiftwidth" when 'softtabstop' is negative]], function()
     local shiftwidth = 2
     globals.curbuf.b_p_sw = shiftwidth
     local tabstop = 5
