@@ -862,6 +862,11 @@ describe('fs.c', function()
     end
 
     describe('os_fileinfo', function()
+      itp('returns false if path=NULL', function()
+        local file_info = file_info_new()
+        assert.is_false((fs.os_fileinfo(nil, file_info)))
+      end)
+
       itp('returns false if given a non-existing file', function()
         local file_info = file_info_new()
         assert.is_false((fs.os_fileinfo('/non-existent', file_info)))
